package cn.com.qmth.stmms.biz.utils;

import java.util.ArrayList;
import java.util.List;

import com.qmth.boot.core.collection.PageResult;

import cn.com.qmth.stmms.biz.common.BaseQuery;

public class PageUtil {

    public static <T, K> PageResult<T> of(List<T> list, BaseQuery<K> query) {
        PageResult<T> pageResult = new PageResult<>();
        pageResult.setResult(list);
        pageResult.setPageNumber(query.getPageNumber());
        pageResult.setPageSize(query.getPageSize());
        pageResult.setTotalCount(query.getTotalCount());
        long count = query.getTotalCount(); // 记录总数
        long pageCount = 0; // 页数
        if (count % query.getPageSize() == 0) {
            pageCount = count / query.getPageSize();
        } else {
            pageCount = count / query.getPageSize() + 1;
        }
        pageResult.setPageCount(pageCount);
        return pageResult;
    }

    public static <T> PageResult<T> of(BaseQuery<T> query) {
        PageResult<T> result = new PageResult<>();
        result.setResult(query.getResult());
        result.setPageNumber(query.getPageNumber());
        result.setPageSize(query.getPageSize());
        result.setTotalCount(query.getTotalCount());
        result.setPageCount(query.getTotalPage());
        return result;
    }

    public static <T> PageResult<T> emptyPage() {
        PageResult<T> page = new PageResult<>();
        page.setPageCount(0);
        page.setPageNumber(1);
        page.setPageSize(10);
        page.setTotalCount(0);
        page.setResult(new ArrayList<>());
        return page;
    }

    public static <T> PageResult<T> of(List<T> list, Integer pageNum, Integer pageSize) {
        if (list == null) {
            return null;
        }
        if (list.size() == 0) {
            return null;
        }

        Integer count = list.size(); // 记录总数
        Integer pageCount = pageCount(list, pageNum, pageSize);

        int fromIndex = 0; // 开始索引
        int toIndex = 0; // 结束索引

        if (pageCount < pageNum) {
            return null;
        }
        if (pageCount - pageNum != 0) {
            fromIndex = (pageNum - 1) * pageSize;
            toIndex = fromIndex + pageSize;
        } else {
            fromIndex = (pageNum - 1) * pageSize;
            toIndex = count;
        }

        List<T> pageList = list.subList(fromIndex, toIndex);
        PageResult<T> page = new PageResult<>();
        page.setPageCount(pageCount);
        page.setPageNumber(pageNum);
        page.setPageSize(pageSize);
        page.setTotalCount(list.size());
        page.setResult(pageList);
        return page;
    }

    private static <T> Integer pageCount(List<T> list, Integer pageNum, Integer pageSize) {
        if (list == null) {
            return 0;
        }
        if (list.size() == 0) {
            return 0;
        }
        Integer count = list.size(); // 记录总数
        Integer pageCount = 0; // 页数
        if (count % pageSize == 0) {
            pageCount = count / pageSize;
        } else {
            pageCount = count / pageSize + 1;
        }

        return pageCount;
    }
}
